@@ -1,31 +1,32 @@
+pub mod calendar;
 pub mod geo_ip;
 pub mod ip_map;
-pub mod ip_net;
-pub mod text_map;
-pub mod ip_set;
-pub mod text_set;
-pub mod calendar;
 pub mod ip_map_list;
+pub mod ip_net;
+pub mod ip_set;
+pub mod text_map;
 pub mod text_map_list;
+pub mod text_set;
 
+use calendar::{CalendarSynDataset, UpdateCalendar};
 use geo_ip::{GeoIpSynDataset, UpdateGeoIp};
 use ip_map::{IpMapSynDataset, UpdateIpMap};
+use ip_map_list::{IpMapListSynDataset, UpdateIpMapList};
 use ip_net::{IpNetSynDataset, UpdateNetIp};
-use text_map::{TextMapSynDataset, UpdateTextMap};
 use ip_set::{IpSetSynDataset, UpdateIpSet};
-use text_set::{TextSetSynDataset, UpdateTextSet};
-use calendar::{CalendarSynDataset, UpdateCalendar};
-use serde::Serialize;
 use serde::ser::{SerializeStruct, Serializer};
+use serde::Serialize;
 use std::borrow::Cow;
-use text_map_list::{UpdateTextMapList, TextMapListSynDataset};
-use ip_map_list::{UpdateIpMapList ,IpMapListSynDataset};
-use std::fmt;
 use std::cmp::Ordering;
+use std::convert::TryFrom;
+use std::fmt;
+use text_map::{TextMapSynDataset, UpdateTextMap};
+use text_map_list::{TextMapListSynDataset, UpdateTextMapList};
+use text_set::{TextSetSynDataset, UpdateTextSet};
 
 /// Common work datasets that allow a rapid development of rules and that the information of some logs allows enriching others.
 /// Other datasets like the ones associated with headquarters is controlled by the CMDB
-/// 
+///
 /// The custom datasets are associated with the name of the dataset
 #[derive(Debug, Clone)]
 #[non_exhaustive]
@@ -83,9 +84,117 @@ pub enum SiemDataset {
     /// Configuration of components. Allows modifications of behaviour in real time.
     Configuration(TextMapSynDataset),
     /// Secret store. A component will only be able to access to his own secrets.
-    Secrets((Cow<'static, str>,TextMapSynDataset))
+    Secrets((Cow<'static, str>, TextMapSynDataset)),
 }
 
+impl TryFrom<(SiemDatasetType, GeoIpSynDataset)> for SiemDataset {
+    type Error = &'static str;
+
+    fn try_from(value: (SiemDatasetType, GeoIpSynDataset)) -> Result<Self, Self::Error> {
+        if value.0 == SiemDatasetType::GeoIp {
+            Err("GeoIPSynDataset is only valid for GeoIP dataset!")
+        } else {
+            Ok(SiemDataset::GeoIp(value.1))
+        }
+    }
+}
+impl TryFrom<(SiemDatasetType, IpMapSynDataset)> for SiemDataset {
+    type Error = &'static str;
+
+    fn try_from(value: (SiemDatasetType, IpMapSynDataset)) -> Result<Self, Self::Error> {
+        match value.0 {
+            SiemDatasetType::IpHost => Ok(SiemDataset::IpHost(value.1)),
+            SiemDatasetType::IpMac => Ok(SiemDataset::IpMac(value.1)),
+            SiemDatasetType::CustomIpMap(name) => {
+                Ok(SiemDataset::CustomIpMap((name.clone(), value.1)))
+            }
+            _ => Err("IpMapSynDataset not valid for this type"),
+        }
+    }
+}
+impl TryFrom<(SiemDatasetType, TextMapSynDataset)> for SiemDataset {
+    type Error = &'static str;
+
+    fn try_from(value: (SiemDatasetType, TextMapSynDataset)) -> Result<Self, Self::Error> {
+        match value.0 {
+            SiemDatasetType::MacHost => Ok(SiemDataset::MacHost(value.1)),
+            SiemDatasetType::HostUser => Ok(SiemDataset::HostUser(value.1)),
+            SiemDatasetType::UserHeadquarters => Ok(SiemDataset::UserHeadquarters(value.1)),
+            SiemDatasetType::Configuration => Ok(SiemDataset::Configuration(value.1)),
+            SiemDatasetType::CustomMapText(name) => {
+                Ok(SiemDataset::CustomMapText((name.clone(), value.1)))
+            }
+            SiemDatasetType::Secrets(name) => Ok(SiemDataset::Secrets((name.clone(), value.1))),
+            _ => Err("TextMapSynDataset not valid for this type"),
+        }
+    }
+}
+impl TryFrom<(SiemDatasetType, IpMapListSynDataset)> for SiemDataset {
+    type Error = &'static str;
+
+    fn try_from(value: (SiemDatasetType, IpMapListSynDataset)) -> Result<Self, Self::Error> {
+        match value.0 {
+            SiemDatasetType::IpDNS => Ok(SiemDataset::IpDNS(value.1)),
+            _ => Err("IpMapSynDataset not valid for this type"),
+        }
+    }
+}
+
+impl TryFrom<(SiemDatasetType, IpSetSynDataset)> for SiemDataset {
+    type Error = &'static str;
+
+    fn try_from(value: (SiemDatasetType, IpSetSynDataset)) -> Result<Self, Self::Error> {
+        match value.0 {
+            SiemDatasetType::BlockIp => Ok(SiemDataset::BlockIp(value.1)),
+            SiemDatasetType::CustomIpList(name) => Ok(SiemDataset::CustomIpList((name.clone(), value.1))),
+            _ => Err("IpMapSynDataset not valid for this type"),
+        }
+    }
+}
+
+impl TryFrom<(SiemDatasetType, TextSetSynDataset)> for SiemDataset {
+    type Error = &'static str;
+
+    fn try_from(value: (SiemDatasetType, TextSetSynDataset)) -> Result<Self, Self::Error> {
+        match value.0 {
+            SiemDatasetType::BlockDomain => Ok(SiemDataset::BlockDomain(value.1)),
+            SiemDatasetType::BlockEmailSender => Ok(SiemDataset::BlockEmailSender(value.1)),
+            SiemDatasetType::BlockCountry => Ok(SiemDataset::BlockCountry(value.1)),
+            SiemDatasetType::CustomTextList(name) => Ok(SiemDataset::CustomTextList((name.clone(), value.1))),
+            _ => Err("IpMapSynDataset not valid for this type"),
+        }
+    }
+}
+
+impl TryFrom<(SiemDatasetType, TextMapListSynDataset)> for SiemDataset {
+    type Error = &'static str;
+
+    fn try_from(value: (SiemDatasetType, TextMapListSynDataset)) -> Result<Self, Self::Error> {
+        match value.0 {
+            SiemDatasetType::HostVulnerable => Ok(SiemDataset::HostVulnerable(value.1)),
+            SiemDatasetType::UserTag => Ok(SiemDataset::UserTag(value.1)),
+            SiemDatasetType::AssetTag => Ok(SiemDataset::AssetTag(value.1)),
+            SiemDatasetType::CustomMapTextList(name) => Ok(SiemDataset::CustomMapTextList((name.clone(), value.1))),
+            _ => Err("IpMapSynDataset not valid for this type"),
+        }
+    }
+}
+
+impl TryFrom<(SiemDatasetType, IpNetSynDataset)> for SiemDataset {
+    type Error = &'static str;
+
+    fn try_from(value: (SiemDatasetType, IpNetSynDataset)) -> Result<Self, Self::Error> {
+        match value.0 {
+            SiemDatasetType::IpCloudService => Ok(SiemDataset::IpCloudService(value.1)),
+            SiemDatasetType::IpCloudProvider => Ok(SiemDataset::IpCloudProvider(value.1)),
+            SiemDatasetType::IpHeadquarters => Ok(SiemDataset::IpHeadquarters(value.1)),
+            SiemDatasetType::CustomMapIpNet(name) => Ok(SiemDataset::CustomMapIpNet((name.clone(), value.1))),
+            _ => Err("IpMapSynDataset not valid for this type"),
+        }
+    }
+}
+
+
 #[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord)]
 #[non_exhaustive]
 pub enum SiemDatasetType {
@@ -145,7 +254,7 @@ pub enum SiemDatasetType {
 impl SiemDataset {
     pub fn dataset_type(&self) -> SiemDatasetType {
         match self {
-            SiemDataset::GeoIp(_) =>SiemDatasetType::GeoIp,
+            SiemDataset::GeoIp(_) => SiemDatasetType::GeoIp,
             SiemDataset::IpHost(_) => SiemDatasetType::IpHost,
             SiemDataset::IpMac(_) => SiemDatasetType::IpMac,
             SiemDataset::IpDNS(_) => SiemDatasetType::IpDNS,
@@ -165,13 +274,15 @@ impl SiemDataset {
             SiemDataset::MantainceCalendar(_) => SiemDatasetType::MantainceCalendar,
             SiemDataset::Configuration(_) => SiemDatasetType::Configuration,
             SiemDataset::HostVulnerable(_) => SiemDatasetType::HostVulnerable,
-            SiemDataset::CustomMapIpNet((name,_)) => SiemDatasetType::CustomMapIpNet(name.clone()),
-            SiemDataset::CustomMapText((name,_)) => SiemDatasetType::CustomMapText(name.clone()),
-            SiemDataset::CustomIpList((name,_)) => SiemDatasetType::CustomIpList(name.clone()),
-            SiemDataset::CustomIpMap((name,_))=> SiemDatasetType::CustomIpMap(name.clone()),
-            SiemDataset::CustomTextList((name,_))=> SiemDatasetType::CustomTextList(name.clone()),
-            SiemDataset::CustomMapTextList((name,_)) => SiemDatasetType::CustomMapTextList(name.clone()),
-            SiemDataset::Secrets((name,_)) => SiemDatasetType::Secrets(name.clone())
+            SiemDataset::CustomMapIpNet((name, _)) => SiemDatasetType::CustomMapIpNet(name.clone()),
+            SiemDataset::CustomMapText((name, _)) => SiemDatasetType::CustomMapText(name.clone()),
+            SiemDataset::CustomIpList((name, _)) => SiemDatasetType::CustomIpList(name.clone()),
+            SiemDataset::CustomIpMap((name, _)) => SiemDatasetType::CustomIpMap(name.clone()),
+            SiemDataset::CustomTextList((name, _)) => SiemDatasetType::CustomTextList(name.clone()),
+            SiemDataset::CustomMapTextList((name, _)) => {
+                SiemDatasetType::CustomMapTextList(name.clone())
+            }
+            SiemDataset::Secrets((name, _)) => SiemDatasetType::Secrets(name.clone()),
         }
     }
 }
@@ -200,7 +311,6 @@ impl Ord for SiemDataset {
     }
 }
 
-
 impl Serialize for SiemDataset {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
@@ -228,31 +338,31 @@ impl Serialize for SiemDataset {
             SiemDataset::MantainceCalendar(_) => "MantainceCalendar",
             SiemDataset::Configuration(_) => "Configuration",
             SiemDataset::HostVulnerable(_) => "HostVulnerable",
-            SiemDataset::CustomMapIpNet((name,_)) => {
+            SiemDataset::CustomMapIpNet((name, _)) => {
                 state.serialize_field("name", name)?;
                 "CustomMapIpNet"
-            },
-            SiemDataset::CustomMapText((name,_)) => {
+            }
+            SiemDataset::CustomMapText((name, _)) => {
                 state.serialize_field("name", name)?;
                 "CustomMapText"
-            },
-            SiemDataset::CustomIpList((name,_)) => {
+            }
+            SiemDataset::CustomIpList((name, _)) => {
                 state.serialize_field("name", name)?;
                 "CustomIpList"
-            },
-            SiemDataset::CustomIpMap((name,_)) => {
+            }
+            SiemDataset::CustomIpMap((name, _)) => {
                 state.serialize_field("name", name)?;
                 "CustomIpMap"
-            },
-            SiemDataset::CustomTextList((name,_)) => {
+            }
+            SiemDataset::CustomTextList((name, _)) => {
                 state.serialize_field("name", name)?;
                 "CustomTextList"
-            },
-            SiemDataset::CustomMapTextList((name,_)) => {
+            }
+            SiemDataset::CustomMapTextList((name, _)) => {
                 state.serialize_field("name", name)?;
                 "CustomMapTextList"
-            },
-            SiemDataset::Secrets((name,_)) => {
+            }
+            SiemDataset::Secrets((name, _)) => {
                 state.serialize_field("name", name)?;
                 "Secrets"
             }
@@ -289,5 +399,5 @@ pub enum UpdateDataset {
     MantainceCalendar(UpdateCalendar),
     Configuration(UpdateTextMap),
     Secrets(UpdateTextMap),
-    HostVulnerable(UpdateTextMapList)
-}
\ No newline at end of file
+    HostVulnerable(UpdateTextMapList),
+}
